//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//  aboutbox.cpp
//  This is a part of the aboutex source code.
//
//  Copyright (C) 1997-2003 The LiteStep Development Team
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

#include "aboutbox.hpp"
#include "main.hpp"
#include "versioning.h"

// items
#include "bangsitem.hpp"
#include "fileitem.hpp"
#include "modulesitem.hpp"
#include "reviditem.hpp"
#include "sysinfoitem.hpp"



//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Globals/Constants
//
const TCHAR* const aboutOptions[] =
{
    _T("Bang Commands"),
    _T("Loaded Modules"),
    _T("Revision IDs"),
    _T("System Information"),
    _T("Changes"),
    _T("Log File"),
};


enum
{
    ABOUT_BANGS,
    ABOUT_MODULES,
    ABOUT_REVIDS,
    ABOUT_SYSINFO,
    ABOUT_CHANGES,
    ABOUT_LOGFILE,
};


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// IMenuItem::Attach
//
void IMenuItem::Attach()
{
    ShowWindow(GetDlgItem(m_hBox, IDC_COMPILETIME), SW_SHOW);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// IMenuItem::Detach
//
void IMenuItem::Detach()
{
    ShowWindow(GetDlgItem(m_hBox, IDC_COMPILETIME), SW_HIDE);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// constructor
//
AboutBox::AboutBox() : m_hBox(NULL), pCurItem(NULL)
{
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// destructor
//
AboutBox::~AboutBox()
{
    pCurItem->Detach();
    delete pCurItem;
}

  
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Show
//
bool AboutBox::Show()
{
    bool bIsOpen = CenterWindow(m_hBox);
    
    if (bIsOpen)
    {
        SwitchToThisWindow(m_hBox, TRUE);
    }

    return bIsOpen;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// OnInitDialog
//
bool AboutBox::OnInitDialog(HWND hBox)
{
    m_hBox = hBox;

    // Set full row selection
    ListView_SetExtendedListViewStyle(_GetListView(), LVS_EX_FULLROWSELECT);
    
    //
    // title
    //
    HFONT titleFont = CreateSimpleFont("Verdana", 14, false);

    SendDlgItemMessage(m_hBox, IDC_TITLE, WM_SETFONT, (WPARAM)titleFont, FALSE);
    
    char szLitestepExe[MAX_PATH] = { 0 };
    GetModuleFileName(GetModuleHandle(NULL), szLitestepExe, MAX_PATH);

    char szBuffer[MAX_PATH] = { "About LiteStep" };
    WORD wVersion[3] = { 0 };
    
    if (GetResourceVersion(szLitestepExe, wVersion, 3))
    {
        StringCchPrintf(szBuffer, MAX_PATH, "LiteStep %u.%u.%u",
            wVersion[0], wVersion[1], wVersion[2]);
    }

    SetDlgItemText(m_hBox, IDC_TITLE, szBuffer);

    //
    // theme info
    //
    char themeAuthor[21] = { 0 };
    char themeName[31] = { 0 };
    char themeOut[MAX_LINE_LENGTH] = { 0 };
    
    GetRCString("ThemeAuthor", themeAuthor, "(unknown)", sizeof(themeAuthor));
    
    if (GetRCString("ThemeName", themeName, NULL, sizeof(themeName)))
    {
        StringCchPrintf(themeOut, MAX_LINE_LENGTH,
            "Theme: %s by %s", themeName, themeAuthor);
    }
    else
    {
        StringCchPrintf(themeOut, MAX_LINE_LENGTH, "Theme by %s", themeAuthor);
    }
    
    SetDlgItemText(m_hBox, IDC_THEME_INFO, themeOut);
    
    //
    // compile time
    //
//    char compileTime[64] = { 0 };
//    StringCchPrintf(compileTime, 64,
//        "Compiled on %s at %s", __DATE__, __TIME__);

//    SetDlgItemText(m_hBox, IDC_COMPILETIME, compileTime);
    SetDlgItemText(m_hBox, IDC_COMPILETIME, "AboutEx 0.1");
    
    //
    // add options to combo box
    //
    for (int i = 0; i < COUNTOF(aboutOptions); i++)
    {
        SendDlgItemMessage(m_hBox, IDC_COMBOBOX, CB_ADDSTRING, 0,
            (LPARAM)aboutOptions[i]);
    }
    
    SendDlgItemMessage(m_hBox, IDC_COMBOBOX, CB_SETCURSEL, ABOUT_REVIDS, 0);
    
    _SetNewItem(ABOUT_REVIDS);
    pCurItem->Attach();
    
    return Show();
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// OnCtlColorStatic
//
HBRUSH AboutBox::OnCtlColorStatic(int id, HDC hDC) const
{
    HBRUSH hReturn = NULL;

    // the header and title need a white (COLOR_WINDOW) background
    if (id == IDC_TITLE || id == IDC_THEME_INFO)
    {
        SetTextColor(hDC, GetSysColor(COLOR_WINDOWTEXT));
        SetBkColor(hDC, GetSysColor(COLOR_WINDOW));
        
        hReturn = GetSysColorBrush(COLOR_WINDOW);
    }
    else if (id == IDC_HEADER || id == IDC_LOGO)
    {
        hReturn = GetSysColorBrush(COLOR_WINDOW);
    }

    return hReturn;    
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// OnCommand
//
bool AboutBox::OnCommand(WORD wNotifyCode, WORD wID)
{
    switch(wID)
    {
        case IDC_COMBOBOX:
        {
            if (wNotifyCode == CBN_SELCHANGE)
            {
                // delete listview items
                SendDlgItemMessage(m_hBox, IDC_LISTVIEW,
                    LVM_DELETEALLITEMS, 0, 0);
                
                // delete listview columns
                for (int i = 3; i >= 0; i--)
                {
                    SendDlgItemMessage(m_hBox, IDC_LISTVIEW,
                        LVM_DELETECOLUMN, (WPARAM) i, 0);
                }
                
                // get new selection
                int j = SendDlgItemMessage(m_hBox, IDC_COMBOBOX,
                    CB_GETCURSEL, 0, 0);

                _SetNewItem(j);
                pCurItem->Attach();
                
                return true;
            }
        }
        break;
        
        case IDOK:
        case IDCANCEL:
        {
            // release title font
            HFONT titleFont = (HFONT)SendDlgItemMessage(m_hBox, IDC_TITLE,
                WM_GETFONT, 0, 0);

            DeleteObject(titleFont);
            
            // close the dialog box
            EndDialog(m_hBox, IDOK);

            return TRUE;
        }

        default:
        break;        
    }

    return false;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// OnNotify
//
bool AboutBox::OnNotify(int nID, LPNMHDR pnmh) const
{
    int iBangLen = 0;
    HWND hListView = _GetListView();
    
    if (pnmh->idFrom == IDC_LISTVIEW)
    {
        if (pnmh->code == LVN_DELETEITEM)
        {
            return true;
        }
        
        int iSelItem = ListView_GetNextItem(hListView, -1, LVNI_SELECTED);
        
        if (ListView_GetSelectedCount(hListView))
        {
            if (pnmh->code == NM_CLICK)
            {
                pCurItem->OnLeftClick(iSelItem);
            }
            else if (pnmh->code == NM_DBLCLK)
            {
                pCurItem->OnLeftDoubleClick(iSelItem);
            }
            else if (pnmh->code == NM_RCLICK)
            {
                pCurItem->OnRightClick(iSelItem);
            }
            else if (pnmh->code == NM_RDBLCLK)
            {
                pCurItem->OnRightDoubleClick(iSelItem);
            }
        }
    }

    return false;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// _SetNewItem
//
bool AboutBox::_SetNewItem(int index)
{
    if (pCurItem)
    {
        pCurItem->Detach();
        delete pCurItem;

        pCurItem = NULL;
    }

    switch (index)
    {
        case ABOUT_BANGS:
        {
            pCurItem = new BangsItem(m_hBox, _GetListView());
        }
        break;

        case ABOUT_MODULES:
        {
            pCurItem = new ModulesItem(m_hBox, _GetListView());
        }
        break;
        
        case ABOUT_REVIDS:
        {
            pCurItem = new RevIDItem(m_hBox, _GetListView());
        }
        break;
        
        case ABOUT_SYSINFO:
        {
            pCurItem = new SysinfoItem(m_hBox, _GetListView());
        }
        break;
        
        case ABOUT_CHANGES:
        {
            char szFile[MAX_PATH] = { 0 };
            
            LSGetLitestepPath(szFile, MAX_PATH-12);
            StringCchCat(szFile, MAX_PATH, "changes.txt");

            pCurItem = new FileItem(m_hBox, _GetListView(), szFile, "Changes");
        }
        break;
        
        case ABOUT_LOGFILE:
        {
            char szFile[MAX_PATH] = { 0 };
            GetRCString("LSLogFile", szFile, NULL, MAX_PATH);

            pCurItem = new FileItem(m_hBox, _GetListView(), szFile, "Log");
        }
        break;
        
        DEFAULT_NOTREACHABLE;
    }

    return pCurItem != NULL;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// AboutBoxProcedure
//
// AboutBox Dialog Procedure
//
BOOL WINAPI AboutBoxProcedure(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
		case WM_INITDIALOG:
		{
            AboutBox* pBox = (AboutBox*)lParam;
            
            if (pBox)
            {
                SetWindowLong(hWnd, GWL_USERDATA, lParam);
                
                return pBox->OnInitDialog(hWnd);
            }
		}

		
		case WM_CTLCOLORSTATIC:
		{
            AboutBox* pBox = (AboutBox*)GetWindowLong(hWnd, GWL_USERDATA);

            if (pBox)
            {
                return (BOOL)pBox->OnCtlColorStatic(GetDlgCtrlID((HWND)lParam),
                    (HDC)wParam);
            }
		}


		case WM_COMMAND:
		{
            AboutBox* pBox = (AboutBox*)GetWindowLong(hWnd, GWL_USERDATA);

            if (pBox)
            {
                return pBox->OnCommand(HIWORD(wParam), LOWORD(wParam));
            }
		}


		case WM_NOTIFY:
		{
            AboutBox* pBox = (AboutBox*)GetWindowLong(hWnd, GWL_USERDATA);

            if (pBox)
            {
                return pBox->OnNotify((int)wParam, (LPNMHDR)lParam);
            }
		}
	}

    return false;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// AboutBoxThread
//
ULONG WINAPI AboutBoxThread(LPVOID lpParam)
{
    static AboutBox* pBox = NULL;
    
    if (!pBox)
    {
        pBox = new AboutBox();

        DialogBoxParam(g_hInstance, MAKEINTRESOURCE(IDD_ABOUTBOX),
            NULL, (DLGPROC)AboutBoxProcedure, (LPARAM)pBox);

        delete pBox;
        pBox = NULL;
    }
    else
    {
        pBox->Show();
    }

	return 0;
}
