//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//  bangsitem.cpp
//  This is a part of the aboutex source code.
//
//  Copyright (C) 1997-2003 The LiteStep Development Team
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

#include "bangsitem.hpp"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Globals/Constants
//
#define IDM_EXECUTEBANG			1010
#define IDM_EXECUTEBANGPARAMS	1011
#define IDM_COPYBANG			1012
#define IDM_COPYBANGPARAMS		1013


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// constructor
//
BangsItem::BangsItem(HWND hBox, HWND hListView)
: IMenuItem(hBox, hListView)
{
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// BangCallback
//
BOOL __stdcall BangCallback(LPCSTR pszName, LPARAM lParam)
{
    CallbackData* pCd = (CallbackData*)lParam;
    
    LVITEM itemInfo;
    itemInfo.mask = LVIF_TEXT;
    itemInfo.iItem = pCd->nItem++;
    itemInfo.pszText = (char*)pszName;
    itemInfo.iSubItem = 0;
    
    ListView_InsertItem(pCd->hListView, &itemInfo);
    
    return TRUE;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Attach
//
void BangsItem::Attach()
{
    ShowWindow(GetDlgItem(m_hBox, IDC_PARAMLABEL), SW_SHOW);
    ShowWindow(GetDlgItem(m_hBox, IDC_BANGPARAMS), SW_SHOW);
    
    LVCOLUMN columnInfo = GetBasicColumn(m_hListView, "Bang Command");
    
    ListView_InsertColumn(m_hListView, 0, &columnInfo);
    
    CallbackData cd = { 0 };
    cd.hListView = m_hListView;
    
    HRESULT hr = EnumLSData(ELD_BANGS, (FARPROC)BangCallback, (LPARAM)&cd);

    if (FAILED(hr))
    {
        char szBuffer[MAX_ERROR] = { 0 };
        GetSystemString(hr, szBuffer, MAX_ERROR);
        
        LVITEM itemInfo;
        itemInfo.mask = LVIF_TEXT;
        itemInfo.iItem = cd.nItem;
        itemInfo.pszText = szBuffer;
        itemInfo.iSubItem = 0;

        ListView_InsertItem(m_hListView, &itemInfo);
    }
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Detach
//
void BangsItem::Detach()
{
    ShowWindow(GetDlgItem(m_hBox, IDC_PARAMLABEL), SW_HIDE);
    ShowWindow(GetDlgItem(m_hBox, IDC_BANGPARAMS), SW_HIDE);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// OnLeftDoubleClick
//
void BangsItem::OnLeftDoubleClick(int iItem)
{
    char szCommand[MAX_BANGCOMMAND] = { 0 };
    char szArgs[MAX_BANGARGS] = { 0 };
    
    ListView_GetItemText(m_hListView, iItem, 0, szCommand, MAX_BANGCOMMAND);
    GetDlgItemText(m_hBox, IDC_BANGPARAMS, szArgs, MAX_BANGARGS);
    
    ParseBangCommand(m_hBox, szCommand, szArgs);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// OnRightClick
//
void BangsItem::OnRightClick(int iItem)
{
    POINT cursorPos;
    GetCursorPos(&cursorPos);
    
    HMENU hMenu = CreatePopupMenu();
    
    const NamedValue<UINT> menuItems[] =
    {
        _T("&Execute"),                     IDM_EXECUTEBANG,
        _T("Execute with &parameters"),     IDM_EXECUTEBANGPARAMS,
        _T("&Copy !bang to clipboard"),     IDM_COPYBANG,
        _T("Copy !bang with p&arameters"),  IDM_COPYBANGPARAMS
    };
    
    for (size_t i = 0; i < COUNTOF(menuItems); ++i)
    {
        AppendMenu(hMenu, MF_STRING, menuItems[i].value, menuItems[i].name);
    }
    
    int menuItem = _TrackPopupMenu(hMenu, cursorPos);
    
    if (menuItem)
    {
        char szBangCommand[MAX_BANGCOMMAND] = { 0 };
        ListView_GetItemText(m_hListView, iItem, 0, szBangCommand,
            MAX_BANGCOMMAND);
        
        switch (menuItem)
        {
            case IDM_COPYBANG:
            {
                CopyToClipboard(m_hBox, szBangCommand);
            }
            break;
            
            case IDM_COPYBANGPARAMS:
            {
                const size_t cchCommand = MAX_BANGCOMMAND + MAX_BANGARGS + 5;
                char szCommand[cchCommand] = { 0 };
                char szBangArgs[MAX_BANGARGS];
                
                GetDlgItemText(m_hBox, IDC_BANGPARAMS, szBangArgs, MAX_BANGARGS);
                
                StringCchPrintf(szCommand, cchCommand, "%s %s",
                    szBangCommand, szBangArgs);
                
                CopyToClipboard(m_hBox, szCommand);
            }
            break;
            
            case IDM_EXECUTEBANG:
            {
                LSExecute(m_hBox, szBangCommand, 0);
            }
            break;
            
            case IDM_EXECUTEBANGPARAMS:
            {
                char szBangArgs[MAX_BANGARGS] = { 0 };
                GetDlgItemText(m_hBox, IDC_BANGPARAMS, szBangArgs, MAX_BANGARGS);
                
                LSExecuteEx(m_hBox, NULL, szBangCommand, szBangArgs, NULL, 0);
            }
            
            DEFAULT_NOTREACHABLE;
        }
    }
}
