//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//  fileitem.cpp
//  This is a part of the aboutex source code.
//
//  Copyright (C) 1997-2003 The LiteStep Development Team
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

#include "fileitem.hpp"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// constructor
//
FileItem::FileItem(HWND hBox, HWND hListView,
                   const std::string& sFile, const std::string& sTitle)
: IMenuItem(hBox, hListView), m_sFile(sFile), m_sTitle(sTitle)
{
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// FileToListView
//
// Inserts the contents of a file into a listview, line by line
//
void FileToListView(HWND hListView, LPCTSTR pszFile)
{
    FILE* f = fopen(pszFile, "r");
    
    if (!f)
    {
        char szBuffer[MAX_LINE_LENGTH] = { 0 };
        
        LVITEM itemInfo = { 0 };
        itemInfo.mask = LVIF_TEXT;
        itemInfo.iItem = 0;
        itemInfo.iSubItem = 0;
        
        itemInfo.pszText = "File not found";
        
        ListView_InsertItem(hListView, &itemInfo);
        return;
    }
    
    fseek(f, 0, SEEK_SET);
    
    char szBuffer[MAX_LINE_LENGTH] = { 0 };
    int i = 0;
    int j = 0;
    int width = GetMaxWidth(hListView);
    
    while (f && !feof(f))
    {
        if (!fgets(szBuffer, MAX_LINE_LENGTH, f))
        {
            break;
        }
        
        int tempWidth = ListView_GetStringWidth(hListView, szBuffer);
        
        if (tempWidth > width)
        {
            width = tempWidth;
            ListView_SetColumnWidth(hListView, 0, width);
        }
        
        LVITEM itemInfo = { 0 };
        
        for (size_t j = 0; szBuffer[j] != 0; ++j) { }
        
        szBuffer[j - 1] = 0;
        
        itemInfo.mask = LVIF_TEXT;
        itemInfo.iItem = i++;
        itemInfo.pszText = szBuffer;
        itemInfo.iSubItem = 0;
        
        ListView_InsertItem(hListView, &itemInfo);
    }
    
    fclose(f);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Attach
//
void FileItem::Attach()
{
    IMenuItem::Attach();

    LVCOLUMN columnInfo = GetBasicColumn(m_hListView, (char*)m_sTitle.c_str());
    
    ListView_InsertColumn(m_hListView, 0, &columnInfo);

    FileToListView(m_hListView, m_sFile.c_str());
}
