//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//  reviditem.cpp
//  This is a part of the aboutex source code.
//
//  Copyright (C) 1997-2003 The LiteStep Development Team
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

#include "reviditem.hpp"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// constructor
//
RevIDItem::RevIDItem(HWND hBox, HWND hListView)
: IMenuItem(hBox, hListView)
{
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// RecIDCallback
//
BOOL __stdcall RevIDCallback(LPCSTR pszRevID, LPARAM lParam)
{
    CallbackData* pCd = (CallbackData*)lParam;
    
    LVITEM itemInfo;
    itemInfo.mask = LVIF_TEXT;
    itemInfo.iItem = pCd->nItem++;
    itemInfo.pszText = (char*)pszRevID;
    itemInfo.iSubItem = 0;
    
    ListView_InsertItem(pCd->hListView, &itemInfo);
    
    return TRUE;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Attach
//
void RevIDItem::Attach()
{
    IMenuItem::Attach();
    
    LVCOLUMN columnInfo = GetBasicColumn(m_hListView, "Revision ID");
    
    ListView_InsertColumn(m_hListView, 0, &columnInfo);
    
    // TODO: might want to insert litestep.exe here
    
    CallbackData cd = { 0 };
    cd.hListView = m_hListView;
    
    HRESULT hr = EnumLSData(ELD_REVIDS, (FARPROC)RevIDCallback, (LPARAM)&cd);
    
    if (FAILED(hr))
    {
        char szBuffer[MAX_ERROR] = { 0 };
        GetSystemString(hr, szBuffer, MAX_ERROR);
        
        LVITEM itemInfo;
        itemInfo.mask = LVIF_TEXT;
        itemInfo.iItem = cd.nItem;
        itemInfo.pszText = szBuffer;
        itemInfo.iSubItem = 0;
        
        ListView_InsertItem(m_hListView, &itemInfo);
    }
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// OnLeftDoubleClick
//
void RevIDItem::OnLeftDoubleClick(int iItem)
{
    char szVersion[MAX_PATH] = { 0 };
    
    ListView_GetItemText(m_hListView, iItem, 0, szVersion, MAX_PATH);

    CopyToClipboard(m_hBox, szVersion);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// OnRightClick
//
void RevIDItem::OnRightClick(int iItem)
{
    POINT cursorPos;
    GetCursorPos(&cursorPos);
    
    HMENU hMenu = CreatePopupMenu();
    
    const NamedValue<UINT> menuItems[] =
    {
        _T("&Copy to clipboard"),  1,
    };
    
    for (size_t i = 0; i < COUNTOF(menuItems); ++i)
    {
        AppendMenu(hMenu, MF_STRING, menuItems[i].value, menuItems[i].name);
    }
    
    int menuItem = _TrackPopupMenu(hMenu, cursorPos);
    
    if (menuItem)
    {
        char szVersion[MAX_PATH] = { 0 };
        ListView_GetItemText(m_hListView, iItem, 0, szVersion, MAX_PATH);
        
        switch (menuItem)
        {
            case 1:
            {
                CopyToClipboard(m_hBox, szVersion);
            }
            break;
            
            DEFAULT_NOTREACHABLE;
        }
    }
    
    DestroyMenu(hMenu);
}
