//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//  sysinfoitem.cpp
//  This is a part of the aboutex source code.
//
//  Copyright (C) 1997-2003 The LiteStep Development Team
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

#include "sysinfoitem.hpp"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// constructor
//
SysinfoItem::SysinfoItem(HWND hBox, HWND hListView)
: IMenuItem(hBox, hListView)
{
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Attach
//
void SysinfoItem::Attach()
{
    IMenuItem::Attach();
    
    //
    // column titles
    //
    LVCOLUMN columnInfo = GetBasicColumn(m_hListView, "Name");
    
    int width = GetMaxWidth(m_hListView);
    columnInfo.cx = width / 2;

	ListView_InsertColumn(m_hListView, 0, &columnInfo);

	columnInfo.cx = width / 2;
	columnInfo.pszText = "Value";
	columnInfo.iSubItem = 1;

	ListView_InsertColumn(m_hListView, 1, &columnInfo);

    //
    // items
    //
    int i = 0;

    LVITEM itemInfo = { 0 };
	itemInfo.mask = LVIF_TEXT;
	itemInfo.iItem = i;
	itemInfo.pszText = "Operating System";
	itemInfo.iSubItem = 0;

	ListView_InsertItem(m_hListView, &itemInfo);

    const size_t cchBuffer = 64;
    char buffer[cchBuffer] = { 0 };
    
    OSVERSIONINFO versionInfo = { 0 };
	versionInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx(&versionInfo);

	if (versionInfo.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
	{
		if (versionInfo.dwMinorVersion >= 90)
        {
            StringCchCopy(buffer, cchBuffer, "Windows ME");
        }
		else if (versionInfo.dwMinorVersion >= 10)
        {
            StringCchCopy(buffer, cchBuffer, "Windows 98");
        }
		else
        {
            StringCchCopy(buffer, cchBuffer, "Windows 95");
        }
	}
	else
	{
		if ((versionInfo.dwMajorVersion == 5) &&
            (versionInfo.dwMinorVersion >= 1))
        {
            StringCchCopy(buffer, cchBuffer, "Windows XP");
        }
		else if (versionInfo.dwMajorVersion == 5)
        {
            StringCchCopy(buffer, cchBuffer, "Windows 2000");
        }
		else
        {
            StringCchPrintf(buffer, cchBuffer, "Windows NT %d.%d",
                versionInfo.dwMajorVersion, versionInfo.dwMinorVersion);
        }
	}

	TrimLeft(versionInfo.szCSDVersion);

	if (versionInfo.szCSDVersion[0])
	{
		StringCchCat(buffer, cchBuffer, " (");
		StringCchCat(buffer, cchBuffer, versionInfo.szCSDVersion);
		StringCchCat(buffer, cchBuffer, ")");
	}

	ListView_SetItemText( m_hListView, i++, 1, buffer );

	// memory information
	MEMORYSTATUS ms = { 0 };
	ms.dwLength = sizeof(MEMORYSTATUS);
	GlobalMemoryStatus(&ms);

	itemInfo.iItem = i;
	itemInfo.pszText = "Memory Load";

	ListView_InsertItem( m_hListView, &itemInfo );

	StringCchPrintf(buffer, cchBuffer, "%d%%", ms.dwMemoryLoad);
	ListView_SetItemText( m_hListView, i++, 1, buffer );

	itemInfo.iItem = i;
	itemInfo.pszText = "Physical Memory Total";

	ListView_InsertItem( m_hListView, &itemInfo );

	FormatBytes(ms.dwTotalPhys, buffer, cchBuffer);
	ListView_SetItemText( m_hListView, i++, 1, buffer );

	itemInfo.iItem = i;
	itemInfo.pszText = "Physical Memory Available";

	ListView_InsertItem( m_hListView, &itemInfo );

	FormatBytes( ms.dwAvailPhys, buffer, cchBuffer);
	ListView_SetItemText( m_hListView, i++, 1, buffer );

	itemInfo.iItem = i;
	itemInfo.pszText = "Swap Space Total";

	ListView_InsertItem( m_hListView, &itemInfo );

	FormatBytes(ms.dwTotalPageFile, buffer, cchBuffer);
	ListView_SetItemText(m_hListView, i++, 1, buffer);

	itemInfo.iItem = i;
	itemInfo.pszText = "Swap Space Available";

	ListView_InsertItem(m_hListView, &itemInfo);

	FormatBytes(ms.dwAvailPageFile, buffer, cchBuffer);
	ListView_SetItemText(m_hListView, i++, 1, buffer);
}
