#ifndef ABOUTEX_UTILITY_HPP_INCLUDED
#define ABOUTEX_UTILITY_HPP_INCLUDED

#if _MSC_VER > 1020
#  pragma once
#endif // _MSC_VER

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//  utility.hpp
//  This is a part of the aboutex source code.
//
//  Copyright (C) 1997-2003 The LiteStep Development Team
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

#include "common.hpp"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// struct CallbackData
//
struct CallbackData
{
    HWND hListView;
    int nItem;
};


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// exception class win32_error
//
class win32_error : public std::runtime_error
{
    mutable std::string m_sWhat;
    DWORD m_dwCode;
    
public:
    win32_error() : runtime_error(""), m_dwCode(GetLastError()) { }
    win32_error(DWORD dwCode) : runtime_error(""), m_dwCode(dwCode) { }
    
    virtual ~win32_error() { }
    
    virtual const char* what() const
    {
        if (m_sWhat.empty())
        {
            LPVOID lpBuffer = NULL;
            
            if (FormatMessage(
                FORMAT_MESSAGE_FROM_SYSTEM |
                FORMAT_MESSAGE_IGNORE_INSERTS |
                FORMAT_MESSAGE_ALLOCATE_BUFFER,
                NULL,
                m_dwCode,
                0,
                (LPSTR)&lpBuffer,
                0,
                NULL
                ))
            {
                m_sWhat.assign((LPCSTR)lpBuffer);
                LocalFree(lpBuffer);
            }
        }
        
        return m_sWhat.c_str();
    }
};


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// function prototypes
//
HFONT CreateSimpleFont(LPCTSTR faceName, int sizeInPoints, bool bold);
LONG GetClientWidth(HWND hWnd);
void CopyToClipboard(HWND hWnd, LPCTSTR pszText);
void FormatBytes(DWORD bytes, LPTSTR buffer, size_t cchBuffer);
void TrimLeft(char* toTrim);
bool GetSystemString(DWORD dwCode, LPTSTR ptzBuffer, size_t cchBuffer);
bool CenterWindow(HWND hWnd);

void Error(const char* pszFormat, ...);

LONG GetMaxWidth(HWND hListView);
LVCOLUMN GetBasicColumn(HWND hListView, LPTSTR pszTitle);


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// DEFAULT_NOTREACHABLE
//
#ifdef _DEBUG
#  define DEFAULT_NOTREACHABLE default: ASSERT(0); break
#elif _MSC_VER >= 1200
#  define DEFAULT_NOTREACHABLE default: __assume(0); break
#else
#  define DEFAULT_NOTREACHABLE default: break
#endif;


#endif // !defined(ABOUTEX_UTILITY_HPP_INCLUDED)
