/********************************************************************
	file:		versioning.cpp
	
	purpose:	utility functions to help identify a file's version
*********************************************************************/


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Includes
//
#include "common.hpp"
#include "versioning.h"
#include <tchar.h>
#include "strsafe.h"


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// GetResourceVersion
//
// Get a file's version information from its version resource.
//
bool GetResourceVersion(IN HINSTANCE hInstance, OUT WORD* lpwVersion, IN UINT uBuffers)
{
	bool bReturn = false;
	
	TCHAR szFile[MAX_PATH];  // FIXME: unicode paths can be longer I think...
	
	// get the filename from the instance handle
    if (GetModuleFileName(hInstance, szFile, MAX_PATH))
	{
		// filename was successfully retrieved, get the version
        bReturn = GetResourceVersion(szFile, lpwVersion, uBuffers);
	}
	
	return bReturn;
}

bool GetResourceVersion(IN LPCTSTR pszFile, OUT WORD* lpwVersion, IN UINT uBuffers)
{
	bool bReturn = false;
	
	// check if the number of buffers is within the supported range
    if (uBuffers > 0 && uBuffers <= 4)
	{
		DWORD dwDummy, dwSize;
		
		// check if the version info can be obtained, and if so check
        // how large the info is
        dwSize = GetFileVersionInfoSize(pszFile, &dwDummy);
		
		if (dwSize)
		{
			// create a buffer matching the info size
            LPBYTE lpBuffer = new BYTE[dwSize];
			
			// finally get the version from the version resource
            if (GetFileVersionInfo(pszFile, NULL, dwSize, lpBuffer))
			{
				LPVOID lpVoid;
				UINT uSize;
				
				// get the version from the main block
                if (VerQueryValue(lpBuffer, _T("\\"), &lpVoid, &uSize))
				{
					VS_FIXEDFILEINFO fixedFileInfo;

					CopyMemory(&fixedFileInfo, lpVoid, uSize);
					
					// insert the info into the appropriate buffers
                    switch (uBuffers)
					{
						case 4:
						{
							lpwVersion[3] = LOWORD(fixedFileInfo.dwFileVersionLS);
						}
						case 3:
						{
							lpwVersion[2] = HIWORD(fixedFileInfo.dwFileVersionLS);
						}
						case 2:
						{
							lpwVersion[1] = LOWORD(fixedFileInfo.dwFileVersionMS);
						}
						case 1:
						{
							lpwVersion[0] = HIWORD(fixedFileInfo.dwFileVersionMS);
						}
					}
					
					bReturn = true;
				}
			}
			
			// free allocated memory
            delete[] lpBuffer;
		}
	}
	
    // clear all buffers in case of failure
    if (!bReturn)
    {
        for (UINT uCounter = 0; uCounter < uBuffers; ++uCounter)
        {
            lpwVersion[uCounter] = 0;
        }
    }

	return bReturn;
}
